"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var css = __importStar(require("css-tree"));
var fs = __importStar(require("fs"));
var Parser = (function () {
    function Parser(inputFilePath, config) {
        this.inputFilePath = inputFilePath;
        this.config = config;
    }
    Parser.prototype.ruleToClass = function (child) {
        var rule = child;
        if (rule.prelude.type === "SelectorList") {
            var selectorList = rule.prelude;
            var selector = selectorList.children[0].children[0];
            if (selector.type === "ClassSelector" ||
                selector.type === "TypeSelector") {
                var classSelector = selector;
                var clazz = { name: "", properties: [] };
                clazz.name = classSelector.name.split("\\:").join(":");
                for (var _i = 0, _a = Array.from(rule.block.children); _i < _a.length; _i++) {
                    var blockChildren = _a[_i];
                    if (blockChildren.type === "Declaration") {
                        var _b = blockChildren, property = _b.property, value = _b.value;
                        clazz.properties.push({
                            name: property,
                            value: this.getValues(property, value)
                        });
                    }
                    else {
                    }
                }
                console.log("returning", clazz.name);
                return clazz;
            }
            else if (selector.type === "TypeSelector") {
                var typeSelector = selector;
            }
            else if (selector.type === "AttributeSelector") {
                var typeSelector = selector;
                return;
            }
            else if (selector.type === "PseudoElementSelector") {
                var typeSelector = selector;
                return;
            }
            else {
                throw new Error("unknown selector type " + selector.type);
            }
        }
    };
    Parser.prototype.getValues = function (property, value) {
        var children = new Array(value.children[0]);
        if (children.length === 1) {
            var child = children[0];
            return this.getValue(child);
        }
        else {
        }
    };
    Parser.prototype.getValue = function (child) {
        if (child.type === "Percentage") {
            return "'" + child.value + "%'";
        }
        else if (child.type === "HexColor") {
            return "'#" + child.value + "'";
        }
        else if (child.type === "Identifier") {
            return "'" + child.name + "'";
        }
        else if (child.type === "Number") {
            return child.value;
        }
        else if (child.type === "Dimension") {
            var dimension = child;
            return "toNative(" + dimension.value + ", \"" + dimension.unit + "\")";
        }
        else {
        }
    };
    Parser.prototype.parse = function () {
        var content = fs.readFileSync(this.inputFilePath);
        var ast = css.parse(content.toString());
        var obj = css.toPlainObject(ast);
        var children = new Array(obj.children);
        var result = new Array();
        var _loop_1 = function (child) {
            if (child.type === "Rule") {
                var clazz_1 = this_1.ruleToClass(child);
                if (clazz_1 !== undefined) {
                    result.push(clazz_1);
                    var breakpoint = "";
                    if (clazz_1.name.indexOf("\\:") !== -1) {
                        breakpoint = clazz_1.name.substring(0, clazz_1.name.indexOf("\\:"));
                        clazz_1.name = clazz_1.name.substring(breakpoint.length, clazz_1.name.length);
                    }
                    if (!breakpoint) {
                        if (this_1.config.platforms) {
                            this_1.config.platforms.map(function (platform) {
                                return result.push({
                                    name: platform + ":" + clazz_1.name,
                                    properties: clazz_1.properties,
                                    condition: "true"
                                });
                            });
                        }
                        if (this_1.config.observeInternetConnectivity) {
                            this_1.config.platforms.map(function (platform) {
                                return result.push({
                                    name: "online:" + clazz_1.name,
                                    properties: clazz_1.properties,
                                    condition: "true"
                                });
                            });
                            this_1.config.platforms.map(function (platform) {
                                return result.push({
                                    name: "offline:" + clazz_1.name,
                                    properties: clazz_1.properties,
                                    condition: "true"
                                });
                            });
                        }
                    }
                }
            }
            else if (child.type === "Atrule") {
                var atrule = child;
                if (atrule.name === "media") {
                    var query = "";
                    var mediaQueryList = new Array(atrule.prelude.children)[0][0];
                    for (var _i = 0, _a = new Array(mediaQueryList.children); _i < _a.length; _i++) {
                        var _mediaQuery = _a[_i];
                        for (var _b = 0, _c = _mediaQuery[0].children; _b < _c.length; _b++) {
                            var _d = _c[_b], name = _d.name, value = _d.value;
                            query = "mediaQuery(\"" + name + ": " + value.value + "px\")";
                        }
                    }
                    if (atrule.block) {
                        for (var _e = 0, _f = Array.from(atrule.block.children); _e < _f.length; _e++) {
                            var rule = _f[_e];
                            var clazz = this_1.ruleToClass(rule);
                            if (clazz) {
                                clazz.condition = "true";
                                result.push(clazz);
                            }
                            else {
                                console.log("is null");
                            }
                        }
                    }
                }
                else {
                    console.log("noo");
                }
            }
            else {
                if (child.type !== "Comment") {
                    console.log("unknown css node", child);
                }
            }
        };
        var this_1 = this;
        for (var _i = 0, _a = children[0]; _i < _a.length; _i++) {
            var child = _a[_i];
            _loop_1(child);
        }
        var f = require("fs").createWriteStream(".log.txt");
        f.write(JSON.stringify(result, null, 2));
        f.close();
        return result;
    };
    return Parser;
}());
exports.Parser = Parser;
