import { createMacro } from "babel-plugin-macros";
import { convertProperty } from "./convertProperty";
import { Parser } from "./Parser";
import { defaultConfig } from "./TailwindConfig";

export default createMacro(macro);

function macro({ references, babel, path }: any) {
  const { default: items = [] } = references;
  const p = require("path").join(process.cwd(), "../", "input.css");
  console.log(`css path: ${p}`);
  const parser = new Parser(p, defaultConfig);
  const parsed = parser.parse();
  items.forEach(
    (reference: {
      parentPath: {
        node: { quasi: { quasis: { value: any }[] } };
        replaceWith: (arg0: any) => void;
      };
    }) => {
      if (!reference.parentPath.node.quasi) {
        return;
      }

      const text = reference.parentPath.node.quasi.quasis[0].value;
      const requestedClasses = text.raw.split(" ").filter(x => x);

      const result = `(() => { 
        let result = {};
        ${requestedClasses
          .map(name => {
            const clazz = parsed.filter(x => x.name === name)[0];
            let classContent = "";
            if (!clazz) {
              console.warn(`unknown tailwind css class "${name}"`);
            } else {
              for (const prop of clazz.properties) {
                if (!clazz.condition) {
                  classContent += `result = {...result, ...{ ${convertProperty({
                    prop,
                    clazz
                  })}}}`;
                } else {
                  classContent += `
                  if (${clazz.condition}) {
                    result = {...result, ...{ ${convertProperty({
                      prop,
                      clazz
                    })}}} 
                  }
                  `;
                }
              }
            }
            return classContent;
          })
          .join("\n")}
        return result;
      })()`;

      (reference.parentPath as any).replaceWithSourceString(result);
      // reference.parentPath.replaceWithSourceString(babel.types.)
    }
  );
}
